"""Top-level package for L2Awinddirection."""

__author__ = """Antoine Grouazel"""
__email__ = 'antoine.grouazel@ifremer.fr'
try:
    from importlib import metadata
except ImportError: # for Python<3.8
    import importlib_metadata as metadata
__version__ = metadata.version('l2awinddirection')



"""
A Grouazel
Oct 2023
<div class="alert alert-block alert-warning">
<b>WARNING:</b>
As the bindings are not properly set up on datarmor-jupyter-hub, it is not possible to process files on
 /home/datawork-cersat-project using the singularity servers available on datarmor-jupyterhub. <br>
<b>Use the validation server</b> to be able to see the files on /home/datawork-cersat-project.
</div>

inspired from generate_L1_wind_product_v2.ipynb (R Marquart)
/raid/singularity-images/tensorflow-22.04.sif

"""
import pdb

import numpy as np
import xarray as xr

# /!\ --------- /!\ ---------- /!\ #
# /!\ ----- M64RN4 class ----!\ #
# !\ --------- /!\ ---------- /!\ #

from l2awinddirection.M64RN4 import M64RN4_distribution


def generate_wind_distribution_product(tiles, m64rn4, nb_classes=36, shape=(44, 44, 1)):
    """
    Generate wind direction from previously extracted tiles and store them in an xarray.Dataset.
    Parameters:
        tiles (xarray.Dataset): L1b path of the file to process.
        model (M64RN4_distribution): list of the M64RN4 models used for prediction.
        shape (tuple of int): shape of the inputs of the M64RN4 models.
    Returns:
        xarray.Dataset: dataset containing the tiles with associated wind direction.
    """
    try:
        tiles_stacked = tiles.stack(all_tiles=["burst", "tile_line", "tile_sample"])
    except:
        tiles_stacked = tiles.stack(all_tiles=["tile_line", "tile_sample"])
    tiles_stacked_no_nan = tiles_stacked.where(
        ~np.any(np.isnan(tiles_stacked.sigma0), axis=(0, 1)), drop=True
    )
    X = tiles_stacked_no_nan.sigma0.transpose("all_tiles", "azimuth", "range").values
    X_normalized = np.array(
        [((x - np.average(x)) / np.std(x)).reshape(shape) for x in X]
    )  # Normalize data

    heading_angle = np.deg2rad(tiles_stacked_no_nan["ground_heading"].values)

    # Assign new coordinates 'bin_centers' to the dataset
    dx = 180 / nb_classes
    angles = np.arange(0 + dx / 2, 180 + dx / 2, dx)

    tiles_stacked_no_nan = tiles_stacked_no_nan.assign_coords(
        bin_centers=("bin_centers", angles)
    )

    # Predict wind direction distributions and add them to the dataset
    # predictions = np.squeeze(m64rn4.model.predict(X_normalized)) # original Robin
    predictions = m64rn4.model.predict(X_normalized)
    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(
        wind_direction_distribution=(["all_tiles", "bin_centers"], predictions)
    )

    # Compute mean wind direction, most likely wind direction and associated standard deviation
    mean_wdir = xr.apply_ufunc(
        compute_mean_direction,
        tiles_stacked_no_nan.wind_direction_distribution,
        tiles_stacked_no_nan.bin_centers,
        input_core_dims=[["bin_centers"], ["bin_centers"]],
        vectorize=True,
    )

    most_likely_wdir = xr.apply_ufunc(
        compute_most_likely_direction,
        tiles_stacked_no_nan.wind_direction_distribution,
        tiles_stacked_no_nan.bin_centers,
        input_core_dims=[["bin_centers"], ["bin_centers"]],
        vectorize=True,
    )

    std_wdir = xr.apply_ufunc(
        compute_standard_deviation,
        tiles_stacked_no_nan.wind_direction_distribution,
        tiles_stacked_no_nan.bin_centers,
        input_core_dims=[["bin_centers"], ["bin_centers"]],
        vectorize=True,
    )

    # Format dataset
    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(
        mean_wdir=(["all_tiles"], mean_wdir.data)
    )
    tiles_stacked_no_nan["mean_wdir"].attrs[
        "definition"
    ] = "180° ambiguous mean wind direction."
    tiles_stacked_no_nan["mean_wdir"].attrs["long_name"] = "Mean wind direction"
    tiles_stacked_no_nan["mean_wdir"].attrs[
        "convention"
    ] = "Clockwise, relative to geographic North."
    tiles_stacked_no_nan["mean_wdir"].attrs["units"] = "degree"
    tiles_stacked_no_nan["mean_wdir"].attrs["vmin"] = 0
    tiles_stacked_no_nan["mean_wdir"].attrs["vmax"] = 180

    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(
        most_likely_wdir=(["all_tiles"], most_likely_wdir.data)
    )
    tiles_stacked_no_nan["most_likely_wdir"].attrs[
        "definition"
    ] = "180° ambiguous most likely wind direction, defined with a bin precision of 2.5°."
    tiles_stacked_no_nan["most_likely_wdir"].attrs[
        "long_name"
    ] = "Most likely wind direction"
    tiles_stacked_no_nan["most_likely_wdir"].attrs[
        "convention"
    ] = "Clockwise, relative to geographic North."
    tiles_stacked_no_nan["most_likely_wdir"].attrs["units"] = "degree"
    tiles_stacked_no_nan["most_likely_wdir"].attrs["vmin"] = 0
    tiles_stacked_no_nan["most_likely_wdir"].attrs["vmax"] = 180

    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(
        std_wdir=(["all_tiles"], std_wdir.data)
    )
    tiles_stacked_no_nan["std_wdir"].attrs[
        "definition"
    ] = "Standard deviation associated with wind direction."
    tiles_stacked_no_nan["std_wdir"].attrs[
        "long_name"
    ] = "Standard deviation of the wind direction"
    tiles_stacked_no_nan["std_wdir"].attrs["units"] = "degree"
    tiles_stacked_no_nan["std_wdir"].attrs["vmin"] = 0
    tiles_stacked_no_nan["std_wdir"].attrs["vmax"] = 180

    return tiles_stacked_no_nan.unstack()


def compute_mean_direction(weights, angles):
    """
    Compute mean direction given an array of weights and angles taking into account the circularity between 0 and 180° of the angles.
    Parameters:
        weights (numpy.ndarray): probability corresponding to each angle.
        angles (numpy.ndarray): array of angles in degrees corresponding to the middle of the bins used for training (e.g.: [0,5] -> 2.5).
    Returns:
        float: Computed mean direction.
    """
    angles_rad = (
        angles * (2 * np.pi) / 180
    )  # Recast angles as radians that range between 0 and 2 pi
    s_a = np.sum(weights * np.sin(angles_rad))
    c_a = np.sum(weights * np.cos(angles_rad))

    mean_direction = np.arctan2(s_a, c_a)

    return (
        mean_direction * 180 / (2 * np.pi)
    )  # Convert back the result to the considered range


def compute_standard_deviation(weights, angles):
    """
    Compute standard deviation given an array of weights and angles taking into account the circularity between 0 and 180° of the angles.
    Parameters:
        weights (numpy.ndarray): probability corresponding to each angle.
        angles (numpy.ndarray): array of angles in degrees corresponding to the middle of the bins used for training (e.g.: [0,5] -> 2.5).
    Returns:
        float: Computed standard deviation.
    """
    mean_dir = compute_mean_direction(weights, angles)

    A = np.abs(angles - mean_dir)
    B = np.abs(180 - np.abs(angles - mean_dir))
    # Select the min difference between the two terms defined above to deal with 180° discontinuity
    delta = np.min(np.vstack([A, B]), axis=0)  # 180° ambiguity
    variance = np.sum(weights * delta ** 2)

    return np.sqrt(variance)


def compute_most_likely_direction(weights, angles):
    """
    Get most likely direction given an array of weights and angles.
    Parameters:
        weights (numpy.ndarray): probability corresponding to each angle.
        angles (numpy.ndarray): array of angles in degrees corresponding to the middle of the bins used for training (e.g.: [0,5] -> 2.5).
    Returns:
        float: Most likely direction corresponding to the highest probability.
    """
    return angles[np.argmax(weights)]



import tensorflow as tf
from tensorflow.keras.layers import Conv2D, BatchNormalization, Activation, Input, ZeroPadding2D, MaxPooling2D, Flatten, \
    Dense, Add, Lambda
from tensorflow.keras import Model
from tensorflow.keras.callbacks import ModelCheckpoint, EarlyStopping
from tensorflow import math, keras

import numpy as np
import scipy as sp
import matplotlib.pyplot as plt
import pytz  # For timezone formating

from datetime import datetime

import l2awinddirection.utils

conf = l2awinddirection.utils.get_conf()

class M64RN4:
    def __init__(self, input_shape, data_augmentation):
        self.input_shape = input_shape
        self.data_augmentation = data_augmentation
        self.callback_path = None

    def load(self, path):
        self.model = keras.models.load_model(path)

    def save(self, path):
        self.model.save(path)

    def create(self):
        pass

    def custom_loss(self):
        pass

    def load_dataset(self, X_path, y_path):

        self.X_raw = np.load(X_path)
        self.y_raw = np.load(y_path)

        self.X_standardized = np.array([(x - np.average(x)) / np.std(x) for x in self.X_raw])

    def get_dataset(self, X_path, y_path):
        pass

    def create_and_compile(self, learning_rate, multi_gpus=False, verbose=False):

        gpus = tf.config.experimental.list_physical_devices('GPU')
        nb_gpus = len(gpus)

        if verbose:
            print("Num GPUs Available: ", nb_gpus)

        optimizer = tf.keras.optimizers.Adam(learning_rate)

        # Divide the training between GPUS if there is more than one
        if (nb_gpus > 1) & multi_gpus:
            strategy = tf.distribute.MirroredStrategy()

            with strategy.scope():
                self.create()
                self.model.compile(optimizer, loss=self.custom_loss())

        else:
            self.create()
            self.model.compile(optimizer, loss=self.custom_loss())

    def launch_training(self, nb_epochs):
        callbacks = self.get_callbacks()
        train_history = self.model.fit(self.X_train, self.y_train,
                                       validation_data=(self.X_test, self.y_test),
                                       epochs=nb_epochs,
                                       shuffle=True,
                                       verbose=1,
                                       callbacks=callbacks)

    def infer(self, patch, norm=False):

        if norm:
            patch = (patch - np.average(patch)) / np.std(patch)

        pred = self.model.predict(patch.reshape(((1,) + self.input_shape)))
        return pred

    def get_callbacks(self):

        date = datetime.now(tz=pytz.timezone("Europe/Paris"))
        filepath = self.callback_path

        tail = '%s/%s/%s' % (self.model.name, datetime.strftime(date, "%d-%m-%y"), datetime.strftime(date, "%Hh%S"))

        if self.data_augmentation:
            filepath = "%s/w_data_augmentation/%s" % (filepath, tail)
        else:
            filepath = filepath = "%s/wo_data_augmentation/%s" % (filepath, tail)

        checkpoint_path = "%s/checkpoints/%s_epoch_{epoch:02d}__loss{val_loss:.2f}.hdf5" % (filepath, tail)
        log_path = "%s/logs" % (filepath)

        # Set up callbacks
        # Checkpoint to save the best model
        checkpoint = ModelCheckpoint(filepath=checkpoint_path,
                                     monitor='val_loss',
                                     verbose=1,
                                     save_best_only=True,
                                     mode='min')

        # Tensorboard callback to keep track of the training history
        tensorboard_callback = tf.keras.callbacks.TensorBoard(log_dir=log_path)

        # Stop the model if no improvement was made after 15 epochs
        early_stopping = EarlyStopping(monitor='val_loss', patience=15)

        return checkpoint, tensorboard_callback, early_stopping

    def RNB_identity(self, X, f=3, channels=64):

        # Save the input value
        X_shortcut = X

        # First Layer
        X = ZeroPadding2D((1, 1))(X)
        X = Conv2D(filters=channels, kernel_size=(f, f), strides=(1, 1), padding="valid")(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        # Second Layer
        X = ZeroPadding2D((1, 1))(X)
        X = Conv2D(filters=channels, kernel_size=(f, f), strides=(1, 1), padding="valid")(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        # Add shortcut value to F(X), and pass it through RELU activation
        X = Add()([X, X_shortcut])
        X = Activation("relu")(X)

        # Max Pooling and Batch Normalization
        X = MaxPooling2D((2, 2), strides=(2, 2))(X)
        X = BatchNormalization()(X)

        return X

    def RNB_convolutionnal(self, X, f=3, channels=64):

        # Save the input value
        X_shortcut = X

        # First Layer
        X = ZeroPadding2D((1, 1))(X)
        X = Conv2D(filters=channels, kernel_size=(f, f), strides=(1, 1), padding="valid")(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        # Second Layer
        X = ZeroPadding2D((1, 1))(X)
        X = Conv2D(filters=channels, kernel_size=(f, f), strides=(1, 1), padding="valid")(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        # Shortcut path
        X_shortcut = ZeroPadding2D((1, 1))(X_shortcut)
        X_shortcut = Conv2D(filters=channels, kernel_size=(f, f), strides=(1, 1), padding="valid")(X_shortcut)
        X_shortcut = BatchNormalization(axis=3)(X_shortcut)

        # Add shortcut value to F(X), and pass it through RELU activation
        X = Add()([X, X_shortcut])
        X = Activation("relu")(X)

        # Max Pooling and Batch Normalization
        X = MaxPooling2D((2, 2), strides=(2, 2))(X)
        X = BatchNormalization()(X)

        return X

    def M64RN4_body(self, X):

        if self.data_augmentation:
            X = Lambda(self.data_augmentation_layer, name="data_augmentation_layer")(X)

        # ResNet blocks
        X = self.RNB_convolutionnal(X)
        X = self.RNB_identity(X)
        X = self.RNB_identity(X)
        X = self.RNB_identity(X)

        # Fully connected network
        X = Flatten()(X)
        X = Dense((512))(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        X = Dense((128))(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        X = Dense((32))(X)
        X = Activation("relu")(X)
        X = BatchNormalization()(X)

        return X

    def data_augmentation_layer(self, x):
        return tf.image.rot90(x, np.random.choice([0, 2]))


class M64RN4_regression(M64RN4):

    def __init__(self, input_shape, data_augmentation):
        super().__init__(input_shape, data_augmentation)
        self.callback_path = conf['callback_path']

    def load_dataset(self, path_X_train, path_y_train, path_X_test, path_y_test):
        self.X_train = np.load(path_X_train)
        self.X_test = np.load(path_X_test)
        self.y_train = np.load(path_y_train)
        self.y_test = np.load(path_y_test)

        print('Shape X_train, X_test : %s, %s \nShape y_train, y_test : %s, %s' % (
        str(self.X_train.shape), str(self.X_test.shape), str(self.y_train.shape), str(self.y_test.shape)))

    def create(self):
        # Define the input shape
        X_input = Input(self.input_shape)
        X = self.M64RN4_body(X_input)

        # Output Layer
        X = Dense((1))(X)

        # Create model
        self.model = Model(inputs=X_input, outputs=X, name='M64RN4')

    def custom_loss(self):
        def regression_loss(y_true, y_pred):
            # Compute 1-cos(y-y_pred)**2
            loss = 1 - math.square(math.cos(math.subtract(y_pred, tf.reshape(y_true, [-1, 1]))))
            return loss

        return regression_loss

    def infer_test_set(self):
        self.y_pred = np.squeeze(self.model.predict(self.X_test, batch_size=64)) % np.pi

    def plot_results(self):
        # y_pred = [self.y_pred[i] for i in range(self.y_pred.shape[0])]
        y_pred = np.rad2deg(self.y_pred % np.pi)
        y_ref = np.rad2deg(self.y_test % np.pi)
        diff = (y_pred - y_ref) % 180

        plt.figure(figsize=(10, 8))
        plt.hist2d(y_ref, y_pred, 100, cmap='OrRd', cmin=1)
        plt.title('Predicted angle in function of the reference angle')
        plt.xlabel('Reference angle (°)')
        plt.ylabel('Angle predicted (°)')
        plt.colorbar(label='Number of values per bins')

        x = np.linspace(0, 360, 1000)
        plt.plot(x, x, color='k')
        plt.plot(x, x - 180, color='k')
        plt.plot(x, x + 180, color='k')

        plt.text(10, 165, 'Difference std = {:.2f}° \nDifference mean = {:.2f}°'.format(sp.stats.circstd(diff, 180),
                                                                                        sp.stats.circmean(diff, 180)),
                 backgroundcolor='white')
        plt.grid(linestyle='--', color='darkgray');


class M64RN4_distribution(M64RN4):

    def __init__(self, input_shape, data_augmentation, n_classes):
        super().__init__(input_shape, data_augmentation)
        # self.callback_path = '/raid/localscratch/?????/models_logs_and_save/distribution'
        self.callback_path = conf['callback_path']
        self.n_classes = n_classes

    def create(self):
        # Define the input shape
        X_input = Input(self.input_shape)
        X = self.M64RN4_body(X_input)

        # Output Layer
        X = Dense((self.n_classes))(X)
        X = Activation(tf.math.softmax)(X)

        # Create model
        self.model = Model(inputs=X_input, outputs=X, name='M64RN4')

    def custom_loss(self):
        return tf.keras.losses.CategoricalCrossentropy



"""
inspired from predict_wind_direction.py (project_rmarquart)
January 2024
"""
import numpy as np
import scipy as sp
import xarray as xr
from scipy import stats
import glob
import os, sys
from tqdm import tqdm
from l2awinddirection.M64RN4 import M64RN4_regression

# --------------------- #
# ----- Functions ----- #
# --------------------- #

def generate_wind_product(tiles, model_regs, shape = (44, 44, 1)):
    """
    Generate wind direction from previously extracted tiles and store them in an xarray.Dataset.
    Parameters:
        tiles (xarray.Dataset): L1b path of the file to process.
        model_regs (list of M64RN4): list of the M64RN4 models used for prediction.
        shape (tuple of int): shape of the inputs of the M64RN4 models.
    Returns:
        (xarray.Dataset): dataset containing the tiles with associated wind direction.
    """
    tiles_stacked = tiles.stack(all_tiles = ['burst', 'tile_line','tile_sample'])
    tiles_stacked_no_nan = tiles_stacked.where(~np.any(np.isnan(tiles_stacked.sigma0), axis = (0, 1)), drop = True)
    X = tiles_stacked_no_nan.sigma0.transpose('all_tiles', 'azimuth', 'range').values
    X_normalized = np.array([((x - np.average(x))/np.std(x)).reshape(shape) for x in X]) # Normalize data
    
    heading_angle = np.deg2rad(tiles_stacked_no_nan['ground_heading'].values) 
    
    #Predict wind directions using all models and obtain mean and std values
    predictions = launch_prediction(X_normalized, model_regs)
    mean_prediction = sp.stats.circmean(predictions, np.pi, axis = 1)
    std_prediction = sp.stats.circstd(predictions, np.pi, axis = 1)
    
    predicted_wdir = mean_prediction + heading_angle
    
    # Format dataset
    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(wind_direction = (['all_tiles'],  np.rad2deg(predicted_wdir)%180))
    tiles_stacked_no_nan['wind_direction'].attrs['definition'] = '180° ambiguous wind direction, clockwise, relative to geographic North.'
    tiles_stacked_no_nan['wind_direction'].attrs['units'] = 'degree'
    tiles_stacked_no_nan['wind_direction'].attrs['vmin'] = 0
    tiles_stacked_no_nan['wind_direction'].attrs['vmax'] = 180
    
    tiles_stacked_no_nan = tiles_stacked_no_nan.assign(wind_std = (['all_tiles'],  np.rad2deg(std_prediction)%180))
    tiles_stacked_no_nan['wind_std'].attrs['definition'] = 'Standard deviation associated with wind direction. Calculated by computing the spread accross the '
    tiles_stacked_no_nan['wind_std'].attrs['units'] = 'degree'
    tiles_stacked_no_nan['wind_std'].attrs['vmin'] = 0
    tiles_stacked_no_nan['wind_std'].attrs['vmax'] = 180

    return tiles_stacked_no_nan.unstack()
    

def launch_prediction(X_normalized, model_regs):
    """
    Predicts wind direction of the given vector.
    Parameters:
        X_normalized (numpy.array): normalized array used for prediction.
        model_regs (list of M64RN4): list of the M64RN4 models used for prediction.
    Returns:
        predictions (numpy.array): array of predictions.
    """
    predictions = np.zeros((X_normalized.shape[0], len(model_regs)))

    for i, m64rn4 in enumerate(model_regs):
        predictions[:, i] = np.squeeze(m64rn4.model.predict(X_normalized))
        
    return predictions


if __name__ == "__main__":

    #Parameters to instantiante the models
    input_shape = (44, 44, 1)
    data_augmentation = True
    learning_rate = 1e-3

    model_regs = []

    #Modify path_best_models depending of the acquisition mode (iw, ew, wv)
    path_best_models = glob.glob('.../analysis/s1_data_analysis/project_rmarquar/wsat/trained_models/iw/*.hdf5')
    for path in path_best_models:

        m64rn4_reg = M64RN4_regression(input_shape, data_augmentation)
        m64rn4_reg.create_and_compile(learning_rate)

        m64rn4_reg.model.load_weights(path)

        model_regs.append(m64rn4_reg)


    #Get listing of files from which to generate the wind direction
    files = glob.glob('.../analysis/s1_data_analysis/project_rmarquar/slc_wind_product/1.4k/*/tiles*1.4k.nc')
    print("Number of files to process: %d" % len(files))

    for file in tqdm(files):
        tiles = xr.open_dataset(file)
        if len(tiles.variables.keys()) == 0:
            continue
        res = generate_wind_product(tiles, model_regs)
        res.to_netcdf(file.replace('.nc', '_wind.nc'))
        # if you want to remove the file containing the tiles used for inference (data kept in final product)
        del tiles
        os.remove(file)


"""Main module.
Step 1: move tiles nc files in a local directory for GPU processing (currently prediction is not working from another volume)
Step 2: do the prediction -> generate a .nc file containing the wind direction
Step 3: move the generated file to the archive storage

"""
import glob
import os
import pdb
import shutil
import xarray as xr
from tqdm import tqdm
import logging
import time
import numpy as np
import warnings

warnings.filterwarnings("ignore")
# logging.getLogger("tensorflow").setLevel(logging.FATAL)
import tensorflow as tf

tf.get_logger().setLevel("ERROR")
logging.getLogger("tensorflow").setLevel(logging.ERROR)
# tf.debugging.set_log_device_placement(True)
# logging.getLogger("tensorflow").setLevel(logging.WARNING)
# tf.logging.set_verbosity(tf.logging.ERROR)
# tf.keras.utils.disable_interactive_logging()
# import keras
# from keras.backend.tensorflow_backend import tf

# tf.keras.utils.disable_interactive_logging()
# import absl.logging
# logging.root.removeHandler(absl.logging._absl_handler)
# absl.logging._warn_preinit_stderr = False

# logger = tf.get_logger()
# logger.setLevel(logging.FATAL)

import l2awinddirection
from l2awinddirection.generate_L2A_winddir_pdf_product import (
    generate_wind_distribution_product,
)
from l2awinddirection.generate_L2A_winddir_regression_product import (
    generate_wind_product,
)
from l2awinddirection.utils import get_conf
from l2awinddirection.M64RN4 import M64RN4_distribution, M64RN4_regression

os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"  # disable logging tensorflow
conf = get_conf()


def get_memory_usage():
    try:
        import resource

        memory_used_go = (
            resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1000.0 / 1000.0
        )
    except:  # on windows resource is not usable
        import psutil

        memory_used_go = psutil.virtual_memory().used / 1000 / 1000 / 1000.0
    str_mem = "RAM usage: %1.1f Go" % memory_used_go
    return str_mem


def main():
    import argparse

    root = logging.getLogger()
    if root.handlers:
        for handler in root.handlers:
            root.removeHandler(handler)
    time.sleep(np.random.rand(1, 1)[0][0])  # to avoid issue with mkdir
    parser = argparse.ArgumentParser(description="winddirection_prediction->L2Awinddir")
    parser.add_argument("--verbose", action="store_true", default=False)
    parser.add_argument(
        "--overwrite",
        action="store_true",
        default=False,
        help="overwrite the existing outputs [default=False]",
        required=False,
    )
    parser.add_argument(
        "--mode",
        choices=["regression", "pdf"],
        help="must must choose a NN model or regression or pdf",
    )
    parser.add_argument(
        "--l2awindirtilessafe",
        required=True,
        help="Level-2A wind direction SAFE where are stored the extracted DN tiles",
    )
    # for now the output files will be in the same directory than the input SAFE
    parser.add_argument(
        "--outputdir",
        required=True,
        help="directory where to move and store output netCDF files",
    )
    parser.add_argument(
        "--workspace",
        help="specify a workspace for tiles I/O [default is the path in localconfig.yml or config.yml]",
        required=False,
    )
    parser.add_argument(
        "--modelpdf",
        help="specify the path of the NN model to predict PDF result [default is localconfig.yml or config.yml]",
        required=False,
    )
    parser.add_argument(
        "--modelregression",
        help="specify the path of the NN model to predict float (regression) result [default is localconfig.yml or config.yml]",
        required=False,
    )
    parser.add_argument(
        "--skipmoves",
        help="skip copy of the tiles in a workspace and move of the L2A winddir files from workspace to outputdir [default=False]",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--remove-tiles",
        help="remove the tiles files in the workspace directory [default=True]",
        required=False,
        default=True,
        action="store_false",
    )
    args = parser.parse_args()
    fmt = "%(asctime)s %(levelname)s %(filename)s(%(lineno)d) %(message)s"
    if args.verbose:
        logging.basicConfig(
            level=logging.DEBUG, format=fmt, datefmt="%d/%m/%Y %H:%M:%S", force=True
        )
    else:
        logging.basicConfig(
            level=logging.INFO, format=fmt, datefmt="%d/%m/%Y %H:%M:%S", force=True
        )
    t0 = time.time()
    logging.info("outputdir will be: %s", args.outputdir)

    # Parameters to instantiante the models
    input_shape = (44, 44, 1)
    data_augmentation = True
    learning_rate = 1e-3
    logging.info("the mode (model) chosen is: %s", args.mode)
    if args.modelpdf is None:
        path_model_pdf = conf["model_path_pdf"]
    else:
        path_model_pdf = args.modelpdf
    if args.modelregression is None:
        dirmodelsreg = conf["models_path_regression"]
    else:
        dirmodelsreg = args.modelregression
    if args.mode == "pdf":
        n_classes = 36
        model_m64rn4 = M64RN4_distribution(input_shape, data_augmentation, n_classes)
        model_m64rn4.create_and_compile(learning_rate)

        model_m64rn4.model.load_weights(path_model_pdf)
    elif args.mode == "regression":
        model_m64rn4 = []
        # path_best_models = glob.glob(
        #     ".../analysis/s1_data_analysis/project_rmarquar/wsat/trained_models/iw/*.hdf5"
        # )
        path_best_models = glob.glob(os.path.join(dirmodelsreg, "*.hdf5"))
        assert len(path_best_models) > 1
        for path in path_best_models:

            m64rn4_reg = M64RN4_regression(input_shape, data_augmentation)
            m64rn4_reg.create_and_compile(learning_rate)

            m64rn4_reg.model.load_weights(path)

            model_m64rn4.append(m64rn4_reg)
    else:
        raise Exception("not handled case")
    # files = glob.glob("/raid/localscratch/agrouaze/tiles_iw_4_wdir/3.1/*SAFE/*.nc")
    if args.l2awindirtilessafe.endswith("/"):

        l2awindirtilessafe = args.l2awindirtilessafe.rstrip("/")
    else:
        l2awindirtilessafe = args.l2awindirtilessafe
    if args.skipmoves:
        workspace = os.path.dirname(l2awindirtilessafe)
        logging.info(
            "workspace is the place where the tiles are already stored: %s (no copies)",
            workspace,
        )
    else:
        if args.workspace is None:
            workspace = conf["workspace_prediction"]
        else:
            workspace = args.workspace
        logging.info(
            "workspace where the tiles will be temporarily moved is: %s",
            workspace,
        )
    safefile = os.path.join(workspace, os.path.basename(l2awindirtilessafe))
    if not os.path.exists(safefile) and args.skipmoves is True:
        logging.info(" step 1: move %s -> %s", l2awindirtilessafe, safefile)
        shutil.copytree(l2awindirtilessafe, safefile)
    files = glob.glob(os.path.join(safefile, "*.nc"))
    logging.info("Number of files to process: %d" % len(files))
    if not os.path.exists(args.outputdir):
        logging.info("mkdir %s", args.outputdir)
        os.makedirs(args.outputdir)

    logging.info("step 2: predictions")
    for ii in tqdm(range(len(files))):
        file = files[ii]
        #     for file in files:
        tiles = xr.open_dataset(file)
        if len(tiles.variables.keys()) == 0:
            logging.info("no data in %s", file)
            continue
        if args.mode == "pdf":
            res = generate_wind_distribution_product(
                tiles, model_m64rn4, nb_classes=36, shape=(44, 44, 1)
            )
            outputfile = file.replace(
                "_wind.nc", "_winddirection_pdf.nc"
            )  # TODO: change to _tiles.nc -> _winddirection.nc
            if args.skipmoves:
                outputfile = outputfile.replace(workspace, args.outputdir)
                if not os.path.exists(os.path.dirname(outputfile)):
                    os.makedirs(os.path.dirname(outputfile))
            res.attrs["winddirection_L2A_processor"] = "l2awinddirection"
            res.attrs["winddirection_L2A_processor_version"] = l2awinddirection.__version__
            res.to_netcdf(outputfile)
        elif args.mode == "regression":

            outputfile = file.replace(
                "_wind.nc", "_winddirection_regression.nc"
            )  # TODO: change to _tiles.nc -> _winddirection.nc
            if args.skipmoves:
                outputfile = outputfile.replace(workspace, args.outputdir)
                if not os.path.exists(os.path.dirname(outputfile)):
                    os.makedirs(os.path.dirname(outputfile))

            res = generate_wind_product(tiles, model_m64rn4)
            res.attrs["winddirection_L2A_processor"] = "l2awinddirection"
            res.attrs["winddirection_L2A_processor_version"] = l2awinddirection.__version__
            res.to_netcdf(outputfile)

        # if you want to remove the file containing the tiles used for inference (data kept in final product)
        del tiles
        if args.remove_tiles and args.skipmoves is False:
            logging.info("remove temporary tiles file in the workspace: %s", file)
            os.remove(file)
    final_safe_path = os.path.join(args.outputdir, os.path.basename(l2awindirtilessafe))
    if args.skipmoves is False:
        logging.info("step 3: move %s -> %s", safefile, final_safe_path)
        shutil.move(safefile, final_safe_path)
    logging.info("Ifremer Level-2A wind direction SAFE path: %s", final_safe_path)
    logging.info("successful SAFE processing")
    logging.info("peak memory usage: %s ", get_memory_usage())
    logging.info("done in %1.3f min", (time.time() - t0) / 60.0)




import l2awinddirection
from yaml import load
import logging
import os
from yaml import CLoader as Loader

def get_conf():
    """

    Returns:
        conf: dict
    """
    local_config_pontential_path = os.path.join(os.path.dirname(l2awinddirection.__file__), 'localconfig.yaml')

    if os.path.exists(local_config_pontential_path):
        config_path = local_config_pontential_path
    else:
        config_path = os.path.join(os.path.dirname(l2awinddirection.__file__), 'config.yaml')
    logging.info('config path: %s', config_path)
    print('config path:', config_path)
    stream = open(config_path, 'r')
    conf = load(stream, Loader=Loader)
    return conf
